@@ -1,4 +1,4 @@
-<?php declare(strict_types=1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
 ** Copyright (C) 2001-2023 Zabbix SIA
@@ -71,7 +71,7 @@ class CControllerHintboxActionlist extends CController {
 
 		$users = $actions['userids']
 			? API::User()->get([
-				'output' => ['alias', 'name', 'surname'],
+				'output' => ['username', 'name', 'surname'],
 				'userids' => array_keys($actions['userids']),
 				'preservekeys' => true
 			])
@@ -89,7 +89,6 @@ class CControllerHintboxActionlist extends CController {
 			'actions' => $actions['actions'],
 			'users' => $users,
 			'mediatypes' => $mediatypes,
-			'config' => select_config(),
 			'foot_note' => ($actions['count'] > ZBX_WIDGET_ROWS)
 				? _s('Displaying %1$s of %2$s found', ZBX_WIDGET_ROWS, $actions['count'])
 				: null
