@@ -77,7 +77,13 @@ class CControllerSearch extends CController {
 			'templates' => [],
 			'total_groups_cnt' => 0,
 			'total_hosts_cnt' => 0,
-			'total_templates_cnt' => 0
+			'total_templates_cnt' => 0,
+			'allowed_ui_hosts' => $this->checkAccess(CRoleHelper::UI_MONITORING_HOSTS),
+			'allowed_ui_conf_hosts' => $this->checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS),
+			'allowed_ui_latest_data' => $this->checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA),
+			'allowed_ui_problems' => $this->checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
+			'allowed_ui_conf_templates' => $this->checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES),
+			'allowed_ui_conf_host_groups' => $this->checkAccess(CRoleHelper::UI_CONFIGURATION_HOST_GROUPS)
 		];
 
 		if ($this->search !== '') {
@@ -107,8 +113,7 @@ class CControllerSearch extends CController {
 			'selectItems' => API_OUTPUT_COUNT,
 			'selectTriggers' => API_OUTPUT_COUNT,
 			'selectGraphs' => API_OUTPUT_COUNT,
-			'selectApplications' => API_OUTPUT_COUNT,
-			'selectScreens' => API_OUTPUT_COUNT,
+			'selectDashboards' => API_OUTPUT_COUNT,
 			'selectHttpTests' => API_OUTPUT_COUNT,
 			'selectDiscoveries' => API_OUTPUT_COUNT,
 			'search' => [
@@ -206,8 +211,6 @@ class CControllerSearch extends CController {
 			'selectItems' => API_OUTPUT_COUNT,
 			'selectTriggers' => API_OUTPUT_COUNT,
 			'selectGraphs' => API_OUTPUT_COUNT,
-			'selectApplications' => API_OUTPUT_COUNT,
-			'selectScreens' => API_OUTPUT_COUNT,
 			'selectHttpTests' => API_OUTPUT_COUNT,
 			'selectDiscoveries' => API_OUTPUT_COUNT,
 			'search' => [
