@@ -110,14 +110,6 @@ else {
 	access_deny();
 }
 
-$tags = getRequest('tags', []);
-foreach ($tags as $key => $tag) {
-	// remove empty new tag lines
-	if ($tag['tag'] === '' && $tag['value'] === '') {
-		unset($tags[$key]);
-	}
-}
-
 // Remove inherited macros data (actions: 'add', 'update' and 'form').
 $macros = cleanInheritedMacros(getRequest('macros', []));
 
@@ -169,153 +161,66 @@ elseif (isset($_REQUEST['clone']) && isset($_REQUEST['hostid'])) {
 	$_REQUEST['form'] = 'clone';
 }
 elseif (hasRequest('add') || hasRequest('update')) {
-	DBstart();
-
-	$save_macros = $macros;
-
-	foreach ($save_macros as &$macro) {
-		unset($macro['allow_revert']);
-	}
-	unset($macro);
-
-	if ($hostid == 0 || $hostPrototype['templateid'] == 0) {
-		$newHostPrototype = [
-			'host' => getRequest('host', ''),
-			'name' => (getRequest('name', '') === '') ? getRequest('host', '') : getRequest('name', ''),
+	try {
+		$input = [
+			'host' => getRequest('host', DB::getDefault('hosts', 'host')),
+			'name' => getRequest(getRequest('name', '') === '' ? 'host' : 'name', DB::getDefault('hosts', 'name')),
+			'custom_interfaces' => getRequest('custom_interfaces', DB::getDefault('hosts', 'custom_interfaces')),
 			'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
 			'discover' => getRequest('discover', DB::getDefault('hosts', 'discover')),
-			'groupLinks' => [],
-			'groupPrototypes' => [],
-			'tags' => $tags,
-			'macros' => $save_macros,
-			'templates' => array_merge(getRequest('templates', []), getRequest('add_templates', [])),
-			'custom_interfaces' => getRequest('custom_interfaces', DB::getDefault('hosts', 'custom_interfaces'))
+			'interfaces' => prepareHostPrototypeInterfaces(
+				getRequest('interfaces', []), getRequest('mainInterfaces', [])
+			),
+			'groupLinks' => prepareHostPrototypeGroupLinks(getRequest('group_links', [])),
+			'groupPrototypes' => prepareHostPrototypeGroupPrototypes(getRequest('group_prototypes', [])),
+			'templates' => zbx_toObject(
+				array_merge(getRequest('templates', []), getRequest('add_templates', [])),
+				'templateid'
+			),
+			'tags' => prepareHostPrototypeTags(getRequest('tags', [])),
+			'macros' => prepareHostPrototypeMacros($macros),
+			'inventory_mode' => getRequest('inventory_mode', HOST_INVENTORY_DISABLED)
 		];
 
-		if (hasRequest('inventory_mode')) {
-			$newHostPrototype['inventory_mode'] = getRequest('inventory_mode');
-		}
+		$result = true;
 
-		// API requires 'templateid' property.
-		if ($newHostPrototype['templates']) {
-			$newHostPrototype['templates'] = zbx_toObject($newHostPrototype['templates'], 'templateid');
-		}
+		if (hasRequest('add')) {
+			$host = ['ruleid' => getRequest('parent_discoveryid')] + getSanitizedHostPrototypeFields(
+				['templateid' => 0] + $input
+			);
 
-		// add custom group prototypes
-		foreach (getRequest('group_prototypes', []) as $groupPrototype) {
-			if ($groupPrototype['name'] !== '') {
-				$newHostPrototype['groupPrototypes'][] = $groupPrototype;
+			$response = API::HostPrototype()->create($host);
+
+			if ($response === false) {
+				throw new Exception();
 			}
 		}
 
-		if ($newHostPrototype['custom_interfaces'] == HOST_PROT_INTERFACES_CUSTOM) {
-			$interfaces = getRequest('interfaces', []);
+		if (hasRequest('update')) {
+			$host = ['hostid' => $hostid] + getSanitizedHostPrototypeFields(
+				['templateid' => $hostPrototype['templateid']] + $input
+			);
 
-			foreach ($interfaces as &$interface) {
-				// Process SNMP interface fields.
-				if ($interface['type'] == INTERFACE_TYPE_SNMP) {
-					$interface['details']['bulk'] = array_key_exists('bulk', $interface['details'])
-						? SNMP_BULK_ENABLED
-						: SNMP_BULK_DISABLED;
+			$response = API::HostPrototype()->update($host);
 
-					switch ($interface['details']['version']) {
-						case SNMP_V1:
-						case SNMP_V2C:
-							$interface['details'] = array_intersect_key($interface['details'],
-								array_flip(['version', 'bulk', 'community'])
-							);
-							break;
-
-						case SNMP_V3:
-							$field_names = array_flip(['version', 'bulk', 'contextname', 'securityname',
-								'securitylevel'
-							]);
-
-							if ($interface['details']['securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV) {
-								$field_names += array_flip(['authprotocol', 'authpassphrase']);
-							}
-							elseif ($interface['details']['securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
-								$field_names +=
-									array_flip(['authprotocol', 'authpassphrase', 'privprotocol', 'privpassphrase']);
-							}
-
-							$interface['details'] = array_intersect_key($interface['details'], $field_names);
-							break;
-					}
-				}
-
-				unset($interface['isNew'], $interface['items'], $interface['interfaceid']);
-				$interface['main'] = 0;
+			if ($response === false) {
+				throw new Exception();
 			}
-			unset($interface);
-
-			$main_interfaces = getRequest('mainInterfaces', []);
-
-			foreach (CItem::INTERFACE_TYPES_BY_PRIORITY as $type) {
-				if (array_key_exists($type, $main_interfaces)
-						&& array_key_exists($main_interfaces[$type], $interfaces)) {
-					$interfaces[$main_interfaces[$type]]['main'] = INTERFACE_PRIMARY;
-				}
-			}
-
-			$newHostPrototype['interfaces'] = $interfaces;
 		}
-	}
-	else {
-		$newHostPrototype = [
-			'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
-			'discover' => getRequest('discover', DB::getDefault('hosts', 'discover'))
-		];
+	} catch (Exception $e) {
+		$result = false;
 	}
 
-	if ($hostid != 0) {
-		$newHostPrototype['hostid'] = $hostid;
-
-		if (!$hostPrototype['templateid']) {
-			// add group prototypes based on existing host groups
-			$groupPrototypesByGroupId = zbx_toHash($hostPrototype['groupLinks'], 'groupid');
-			unset($groupPrototypesByGroupId[0]);
-			foreach (getRequest('group_links', []) as $groupId) {
-				$newHostPrototype['groupLinks'][] = [
-					'groupid' => array_key_exists($groupId, $groupPrototypesByGroupId)
-						? $groupPrototypesByGroupId[$groupId]['groupid']
-						: $groupId
-				];
-
-			}
-		}
-		else {
-			unset($newHostPrototype['groupPrototypes'], $newHostPrototype['groupLinks']);
-		}
-
-		$result = API::HostPrototype()->update($newHostPrototype);
-
-		show_messages($result, _('Host prototype updated'), _('Cannot update host prototype'));
-	}
-	else {
-		$newHostPrototype['ruleid'] = getRequest('parent_discoveryid');
-
-		// add group prototypes based on existing host groups
-		foreach (getRequest('group_links', []) as $groupId) {
-			$newHostPrototype['groupLinks'][] = [
-				'groupid' => $groupId
-			];
-		}
-
-		if (count($newHostPrototype['groupPrototypes']) === 0) {
-			unset($newHostPrototype['groupPrototypes']);
-		}
-
-		$result = API::HostPrototype()->create($newHostPrototype);
-
+	if (hasRequest('add')) {
 		show_messages($result, _('Host prototype added'), _('Cannot add host prototype'));
 	}
-
-	$result = DBend($result);
+	else {
+		show_messages($result, _('Host prototype updated'), _('Cannot update host prototype'));
+	}
 
 	if ($result) {
-		uncheckTableRows($discoveryRule['itemid']);
 		unset($_REQUEST['itemid'], $_REQUEST['form']);
+		uncheckTableRows($discoveryRule['itemid']);
 	}
 }
 elseif ($hostid != 0 && getRequest('action', '') === 'hostprototype.updatediscover') {
@@ -403,7 +308,7 @@ if (hasRequest('form')) {
 		'show_inherited_macros' => getRequest('show_inherited_macros', 0),
 		'readonly' => ($hostid != 0 && $hostPrototype['templateid']),
 		'groups' => [],
-		'tags' => $tags,
+		'tags' => getRequest('tags', []),
 		'context' => getRequest('context'),
 		// Parent discovery rules.
 		'templates' => []
