@@ -309,8 +309,9 @@ abstract class CHostGeneral extends CHostBase {
 	 *
 	 * @param array      $hosts
 	 * @param array|null $db_hosts
+	 * @param array|null $upd_hostids
 	 */
-	protected function updateTemplates(array &$hosts, array $db_hosts = null): void {
+	protected function updateTemplates(array &$hosts, ?array $db_hosts = null, array &$upd_hostids = null): void {
 		$id_field_name = $this instanceof CTemplate ? 'templateid' : 'hostid';
 
 		parent::updateTemplates($hosts, $db_hosts);
@@ -720,38 +721,7 @@ abstract class CHostGeneral extends CHostBase {
 
 		// host prototypes
 		if (!$clear && $upd_items[ZBX_FLAG_DISCOVERY_RULE]) {
-			$host_prototypes = DBSelect(
-				'SELECT DISTINCT h.hostid,h3.status as host_status'.
-				' FROM hosts h'.
-					' INNER JOIN host_discovery hd ON h.hostid=hd.hostid'.
-					' INNER JOIN hosts h2 ON h.templateid=h2.hostid'.
-					' INNER JOIN host_discovery hd2 ON h.hostid=hd.hostid'.
-					' INNER JOIN items i ON hd.parent_itemid=i.itemid'.
-					' INNER JOIN hosts h3 ON i.hostid=h3.hostid'.
-				' WHERE '.dbConditionInt('hd.parent_itemid', array_keys($upd_items[ZBX_FLAG_DISCOVERY_RULE]))
-			);
-
-			$upd_host_prototypes = [];
-
-			while ($host_prototype = DBfetch($host_prototypes)) {
-				$upd_host_prototype = ['templateid' => 0];
-				if ($host_prototype['host_status'] == HOST_STATUS_TEMPLATE) {
-					$upd_host_prototype['uuid'] = generateUuidV4();
-				}
-
-				$upd_host_prototypes[$host_prototype['hostid']] = [
-					'values' => $upd_host_prototype,
-					'where' => ['hostid' => $host_prototype['hostid']]
-				];
-			}
-
-			if ($upd_host_prototypes) {
-				DB::update('hosts', $upd_host_prototypes);
-				DB::update('group_prototype', [
-					'values' => ['templateid' => 0],
-					'where' => ['hostid' => array_keys($upd_host_prototypes)]
-				]);
-			}
+			API::HostPrototype()->unlinkTemplateObjects(array_keys($upd_items[ZBX_FLAG_DISCOVERY_RULE]));
 		}
 
 		// http tests
@@ -823,7 +793,7 @@ abstract class CHostGeneral extends CHostBase {
 
 		if ($ruleids) {
 			API::ItemPrototype()->syncTemplates($link_request);
-			API::HostPrototype()->syncTemplates($ruleids, $hostids);
+			API::HostPrototype()->linkTemplateObjects($ruleids, $hostids);
 		}
 
 		API::Trigger()->syncTemplates($link_request);
@@ -1056,7 +1026,7 @@ abstract class CHostGeneral extends CHostBase {
 
 			if ($ruleids) {
 				API::ItemPrototype()->syncTemplates($link_request);
-				API::HostPrototype()->syncTemplates($ruleids, $link_request['hostids']);
+				API::HostPrototype()->linkTemplateObjects($ruleids, $link_request['hostids']);
 			}
 		}
 
@@ -1382,37 +1352,8 @@ abstract class CHostGeneral extends CHostBase {
 		// host prototypes
 		// we need only to unlink host prototypes. in case of unlink and clear they will be deleted together with LLD rules.
 		if (!$clear && $upd_items[ZBX_FLAG_DISCOVERY_RULE]) {
-			$host_prototypes = DBSelect(
-				'SELECT DISTINCT h.hostid,h3.status as host_status'.
-				' FROM hosts h'.
-					' INNER JOIN host_discovery hd ON h.hostid=hd.hostid'.
-					' INNER JOIN hosts h2 ON h.templateid=h2.hostid'.
-					' INNER JOIN host_discovery hd2 ON h.hostid=hd.hostid'.
-					' INNER JOIN items i ON hd.parent_itemid=i.itemid'.
-					' INNER JOIN hosts h3 ON i.hostid=h3.hostid'.
-				' WHERE '.dbConditionInt('hd.parent_itemid', array_keys($upd_items[ZBX_FLAG_DISCOVERY_RULE]))
-			);
-
-			$upd_host_prototypes = [];
-
-			while ($host_prototype = DBfetch($host_prototypes)) {
-				$upd_host_prototype = ['templateid' => 0];
-				if ($host_prototype['host_status'] == HOST_STATUS_TEMPLATE) {
-					$upd_host_prototype['uuid'] = generateUuidV4();
-				}
-
-				$upd_host_prototypes[$host_prototype['hostid']] = [
-					'values' => $upd_host_prototype,
-					'where' => ['hostid' => $host_prototype['hostid']]
-				];
-			}
-
-			if ($upd_host_prototypes) {
-				DB::update('hosts', $upd_host_prototypes);
-				DB::update('group_prototype', [
-					'values' => ['templateid' => 0],
-					'where' => ['hostid' => array_keys($upd_host_prototypes)]
-				]);
+			if (!$clear && $upd_items[ZBX_FLAG_DISCOVERY_RULE]) {
+				API::HostPrototype()->unlinkTemplateObjects(array_keys($upd_items[ZBX_FLAG_DISCOVERY_RULE]));
 			}
 		}
 
