@@ -1,4 +1,4 @@
-<?php declare(strict_types = 1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
 ** Copyright (C) 2001-2023 Zabbix SIA
@@ -185,31 +185,20 @@ class CMenuItem extends CTag {
 	 *
 	 * @return bool  True, if menu item was selected.
 	 */
-	public function setSelectedByAction(string $action_name, array $request_params, bool $expand = true): bool {
+	public function setSelectedByAction(string $action_name, array $request_params, bool $expand = false): bool {
 		if (array_key_exists($action_name, $this->aliases)) {
 			foreach ($this->aliases[$action_name] as $alias_params) {
-				$has_unacceptable_params = false;
 				$has_mandatory_params = true;
 
 				foreach ($alias_params as $name => $value) {
-					if ($name[0] === '!') {
-						$name = substr($name, 1);
-						if (array_key_exists($name, $request_params)
-								&& ($value === '*' || $value === $request_params[$name])) {
-							$has_unacceptable_params = true;
-							break;
-						}
-					}
-					else {
-						if (!array_key_exists($name, $request_params)
-								|| ($value !== '*' && $value !== $request_params[$name])) {
-							$has_mandatory_params = false;
-							break;
-						}
+					if (!array_key_exists($name, $request_params)
+							|| ($value !== '*' && $value !== $request_params[$name])) {
+						$has_mandatory_params = false;
+						break;
 					}
 				}
 
-				if (!$has_unacceptable_params && $has_mandatory_params) {
+				if ($has_mandatory_params) {
 					$this->setSelected();
 
 					return true;
