@@ -69,9 +69,9 @@ class CPageHeader {
 	 * @param string $lang
 	 */
 	public function __construct(string $title, string $lang) {
-		$this->title = $title;
+		$this->title = CHtml::encode($title);
 		$this->lang = $lang;
-		$this->sid = substr(get_cookie(ZBX_SESSION_NAME), 16, 16);
+		$this->sid = substr(CSessionHelper::getId(), 16, 16);
 	}
 
 	/**
@@ -129,16 +129,14 @@ class CPageHeader {
 	 */
 	public function display() {
 		echo '<!DOCTYPE html>'."\n";
-		echo (new CTag('html'))->setAttribute('lang', $this->lang);
+		echo '<html lang="'.$this->lang.'">'."\n";
 		echo <<<HTML
 	<head>
 		<meta http-equiv="X-UA-Compatible" content="IE=Edge"/>
 		<meta charset="utf-8" />
 		<meta name="viewport" content="width=device-width, initial-scale=1">
 		<meta name="Author" content="Zabbix SIA" />
-HTML;
-		echo (new CTag('title', true))->addItem($this->title);
-		echo <<<HTML
+		<title>$this->title</title>
 		<link rel="icon" href="favicon.ico">
 		<link rel="apple-touch-icon-precomposed" sizes="76x76" href="assets/img/apple-touch-icon-76x76-precomposed.png">
 		<link rel="apple-touch-icon-precomposed" sizes="120x120" href="assets/img/apple-touch-icon-120x120-precomposed.png">
@@ -157,10 +155,7 @@ HTML;
 				$path .= '?'.(int) filemtime($path);
 			}
 
-			echo (new CTag('link'))
-				->setAttribute('rel', 'stylesheet')
-				->setAttribute('type', 'text/css')
-				->setAttribute('href', $path);
+			echo '<link rel="stylesheet" type="text/css" href="'.htmlspecialchars($path).'" />'."\n";
 		}
 
 		if ($this->styles) {
@@ -180,7 +175,7 @@ HTML;
 				$path .= '?'.(int) filemtime($path);
 			}
 
-			echo (new CTag('script', true))->setAttribute('src', $path);
+			echo '<script src="'.htmlspecialchars($path).'"></script>'."\n";
 		}
 
 		if ($this->js) {
